"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformQuerySchema = void 0;
const transformQuerySchemaJson = require("../schema/transform-query-schema.json");
function transformQuerySchema(entity, fieldsArray, relationsField, schemaName, arrayPropsConfig) {
    const json = JSON.parse(JSON.stringify(transformQuerySchemaJson));
    json.$id = `${json.$id}/${schemaName}`;
    const fieldSchema = fieldsArray.reduce((acum, item) => {
        acum['sort'][item] = {
            $ref: '#/$defs/sortSchema',
        };
        if (arrayPropsConfig.arrayProps[item]) {
            acum['filter'][item] = {
                $ref: '#/$defs/operandForArrayProps',
            };
        }
        else {
            acum['filter'][item] = {
                $ref: '#/$defs/operand',
            };
        }
        acum['fields'].push(item);
        return acum;
    }, {
        sort: {},
        fields: [],
        filter: {},
    });
    const relationSchema = Object.entries(relationsField).reduce((acum, [key, value]) => {
        fieldSchema['filter'][key] = {
            $ref: '#/$defs/operandForRelationProps',
        };
        acum['sort'][key] = {
            type: 'object',
            additionalProperties: false,
            minProperties: 1,
            properties: value.reduce((a, i) => {
                a[i] = {
                    $ref: '#/$defs/sortSchema',
                };
                return a;
            }, {}),
        };
        acum['fields'][key] = {
            type: 'array',
            items: {
                type: 'string',
                enum: value,
            },
        };
        acum['filter'][key] = {
            type: 'object',
            additionalProperties: false,
            minProperties: 1,
            properties: value.reduce((a, i) => {
                if (arrayPropsConfig.relationArrayProps[key][i]) {
                    a[i] = {
                        $ref: '#/$defs/operandForArrayProps',
                    };
                }
                else {
                    a[i] = {
                        $ref: '#/$defs/operand',
                    };
                }
                return a;
            }, {}),
        };
        return acum;
    }, {
        sort: {},
        fields: {},
        filter: {},
    });
    if (Object.keys(relationsField).length === 0) {
        delete json.$defs['includeDefs'].items.enum;
    }
    else {
        json.$defs['includeDefs'].items.enum.push(...Object.keys(relationsField));
    }
    json.$defs['sortDefs'].properties = Object.assign(Object.assign({}, json.$defs['sortDefs'].properties), Object.assign({ target: {
            type: 'object',
            additionalProperties: false,
            minProperties: 1,
            properties: Object.assign({}, fieldSchema.sort),
        } }, relationSchema.sort));
    json.$defs['fieldsDefs'].properties = Object.assign(Object.assign(Object.assign({}, json.$defs['fieldsDefs'].properties), {
        target: {
            type: 'array',
            items: {
                type: 'string',
                enum: fieldSchema.fields,
            },
        },
    }), relationSchema.fields);
    json.$defs['filterTarget'].properties = Object.assign(Object.assign({}, json.$defs['filterTarget'].properties), fieldSchema.filter);
    json.$defs['filterRelation'].properties = Object.assign(Object.assign({}, json.$defs['filterRelation'].properties), relationSchema.filter);
    return json;
}
exports.transformQuerySchema = transformQuerySchema;
//# sourceMappingURL=transform-query-schema.js.map