"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BodyInputPatchPipe = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const ajv_1 = require("ajv");
const helper_1 = require("../../../helper");
const class_validator_1 = require("class-validator");
class BodyInputPatchPipe {
    constructor(repository, ajvCall) {
        this.repository = repository;
        this.ajvCall = ajvCall;
        this.relationList = this.repository.metadata.relations.reduce((acum, field) => (acum.add(field.propertyName), acum), new Set());
        const schemaName = (0, helper_1.getEntityName)(this.repository.target);
        this.validateFunction = this.ajvCall.getSchema(`inputBodyPatchSchema-${schemaName}`);
    }
    static inject(pip) {
        (0, common_1.Inject)(ajv_1.default)(pip, 'ajvCall', 1);
    }
    transform(value) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const validate = this.validateFunction(value);
            const errorResult = [];
            if (!validate) {
                for (const error of this.validateFunction.errors) {
                    const errorMsg = [];
                    errorMsg.push((0, helper_1.upperFirstLetter)(error.message));
                    switch (error.keyword) {
                        case 'enum':
                            errorMsg.push(`Allowed values are: "${error.params.allowedValues.join(',')}"`);
                            break;
                        case 'additionalProperties':
                            errorMsg.push(`Additional Property is: "${error.params.additionalProperty}"`);
                            break;
                    }
                    errorResult.push({
                        source: {
                            parameter: error.instancePath,
                        },
                        detail: errorMsg.join('. '),
                    });
                }
            }
            if (errorResult.length > 0) {
                throw new common_1.UnprocessableEntityException(errorResult);
            }
            const temporaryEntity = Object.assign(
            // @ts-ignore
            new this.repository.target(), value.data.attributes, Object.keys(value.data.relationships || {}).reduce((acum, key) => {
                acum[key] = value.data.relationships[key].data;
                return acum;
            }, {}));
            const validationErrors = yield (0, class_validator_1.validate)(temporaryEntity, {
                skipUndefinedProperties: true,
            });
            if (validationErrors.length > 0) {
                for (const errorItem of validationErrors) {
                    const errorsList = Object.values(errorItem.constraints).map((message) => {
                        const error = {
                            source: { pointer: `/data/attributes/${errorItem.property}` },
                            detail: (0, helper_1.upperFirstLetter)(message),
                        };
                        if (this.relationList.has(errorItem.property)) {
                            error.source.pointer = `/data/relationships/${errorItem.property}`;
                        }
                        return error;
                    });
                    errorResult.push(...errorsList);
                }
            }
            if (errorResult.length > 0) {
                throw new common_1.UnprocessableEntityException(errorResult);
            }
            const dateKey = Object.keys(value.data.attributes).filter((i) => Reflect.getMetadata('design:type', this.repository.target['prototype'], i) === Date);
            for (const dateField of dateKey) {
                value.data.attributes[dateField] = new Date(value.data.attributes[dateField]);
            }
            return value.data;
        });
    }
}
exports.BodyInputPatchPipe = BodyInputPatchPipe;
//# sourceMappingURL=body-input-patch.pipe.js.map