"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformMixinService = void 0;
const common_1 = require("@nestjs/common");
const route_path_factory_1 = require("@nestjs/core/router/route-path-factory");
const class_transformer_1 = require("class-transformer");
const helper_1 = require("../../../helper");
class TransformMixinService {
    constructor(repository, config, applicationConfig) {
        this.repository = repository;
        this.config = config;
        this.applicationConfig = applicationConfig;
        this.currentResourceName = (0, helper_1.snakeToCamel)(this.repository.metadata.name);
        this.currentPrimaryField = this.repository.metadata.primaryColumns[0].propertyName;
        this.relationPrimaryField = this.repository.metadata.relations.reduce((acum, item) => {
            const relationMetadata = this.repository.manager.getRepository(item.inverseEntityMetadata.target).metadata;
            acum.set(item.propertyName, relationMetadata.primaryColumns[0].propertyName);
            return acum;
        }, new Map());
        this.relationColumns = this.repository.metadata.relations.reduce((a, i) => (a.add(i.propertyName), a), new Set());
        this.commonColumns = this.repository.metadata.columns
            .map((i) => i.propertyName)
            .filter((i) => !this.relationColumns.has(i))
            .reduce((a, i) => (a.add(i), a), new Set());
        this.commonColumnsForRelation = this.repository.metadata.relations.reduce((acum, item) => {
            const relationMetadata = this.repository.manager.getRepository(item.inverseEntityMetadata.target).metadata;
            const relation = relationMetadata.relations.map((i) => i.propertyName);
            const columns = relationMetadata.columns
                .filter((i) => !relation.includes(i.propertyName))
                .reduce((a, i) => (a.add(i.propertyName), a), new Set());
            acum.set(item.propertyName, columns);
            return acum;
        }, new Map());
        this.relationColumnsForRelation = this.repository.metadata.relations.reduce((acum, item) => {
            const relationMetadata = this.repository.manager.getRepository(item.inverseEntityMetadata.target).metadata;
            const columns = relationMetadata.relations.reduce((a, i) => (a.add(i.propertyName), a), new Set());
            acum.set(item.propertyName, columns);
            return acum;
        }, new Map());
        this.relationTarget = this.repository.metadata.relations.reduce((acum, item) => {
            acum[item.propertyName] = item.inverseEntityMetadata.target;
            acum.set(item.propertyName, item.inverseEntityMetadata.target);
            return acum;
        }, new Map());
        this.isArrayRelations = this.repository.metadata.relations.reduce((acum, i) => {
            switch (i.relationType) {
                case 'one-to-one':
                case 'many-to-one':
                    acum[i.propertyName] = false;
                    break;
                case 'many-to-many':
                case 'one-to-many':
                    acum[i.propertyName] = true;
                    break;
            }
            return acum;
        }, {});
    }
    transformRawData(data) {
        const countData = data.length;
        const dataJson = {};
        const currentRawPrimaryField = `${this.currentResourceName}_${this.currentPrimaryField}`;
        for (let i = 0; i < countData; i++) {
            const currentItem = data[i];
            const currentId = currentItem[currentRawPrimaryField].toString();
            dataJson[currentId] = dataJson[currentId] || {};
            const mainObject = {};
            const relationObject = {};
            for (const field in currentItem) {
                const [tableName, ...other] = field.split('_');
                const fieldName = (0, helper_1.snakeToCamel)(other.join('_'));
                if (tableName === this.currentResourceName &&
                    this.commonColumns.has(fieldName)) {
                    mainObject[fieldName] = currentItem[field];
                }
                if (this.commonColumnsForRelation.has(tableName) &&
                    this.commonColumnsForRelation.get(tableName).has(fieldName)) {
                    if (!(field in currentItem)) {
                        continue;
                    }
                    relationObject[tableName] = relationObject[tableName] || {};
                    relationObject[tableName][fieldName] = currentItem[field];
                }
            }
            dataJson[currentId] = Object.assign(Object.assign(Object.assign({}, dataJson[currentId]), mainObject), Object.entries(relationObject).reduce((acum, [key, val]) => {
                if (!acum[key]) {
                    acum[key] = this.isArrayRelations[key] ? [] : {};
                }
                const plainObject = (0, class_transformer_1.plainToClass)(this.relationTarget.get(key), val);
                if (Array.isArray(acum[key])) {
                    if (plainObject[this.relationPrimaryField.get(key)] !== null) {
                        acum[key].push(plainObject);
                    }
                }
                else {
                    acum[key] =
                        plainObject[this.relationPrimaryField.get(key)] === null
                            ? undefined
                            : plainObject;
                }
                return acum;
            }, dataJson[currentId]));
        }
        return Object.values(dataJson).map((i) => (0, class_transformer_1.plainToClass)(this.repository.target, i));
    }
    transformData(data, include = [], table = this.currentResourceName) {
        const urlTable = (0, helper_1.camelToKebab)(table);
        const attributes = {};
        const relationships = {};
        const relationList = table === this.currentResourceName
            ? this.relationColumns
            : this.relationColumnsForRelation.get(table);
        for (const field in data) {
            if (field !== 'id' && !relationList.has(field)) {
                attributes[field] = data[field];
            }
            if (relationList.has(field)) {
                const builtData = {
                    data: undefined,
                };
                const propsData = data[field];
                const typeName = (0, helper_1.getEntityName)(this.relationTarget.get(field));
                if (Array.isArray(propsData)) {
                    const tmp = propsData.reduce((acum, item) => {
                        const key = `${(0, helper_1.camelToKebab)(typeName)}:${item[this.relationPrimaryField.get(field)].toString()}`;
                        if (!acum.has(key)) {
                            acum.set(key, {
                                type: (0, helper_1.camelToKebab)(typeName),
                                id: item[this.relationPrimaryField.get(field)].toString(),
                            });
                        }
                        return acum;
                    }, new Map());
                    builtData.data = [...tmp.values()];
                }
                if (!Array.isArray(data[field]) && typeof data[field] !== 'undefined') {
                    builtData.data = {
                        type: (0, helper_1.camelToKebab)(typeName),
                        id: data[field][this.relationPrimaryField.get(field)].toString(),
                    };
                }
                relationships[field] = Object.assign(Object.assign({}, (include.includes(field) && builtData.data ? builtData : {})), { links: {
                        self: this.getLink(urlTable, data[this.relationPrimaryField.get(field)], 'relationships', (0, helper_1.camelToKebab)(field)),
                        related: this.getLink(urlTable, data[this.relationPrimaryField.get(field)], (0, helper_1.camelToKebab)(field)),
                    } });
            }
        }
        const idNameField = table === this.currentResourceName
            ? this.currentPrimaryField
            : this.relationPrimaryField.get(table);
        for (const itemRelation of relationList.values()) {
            const field = (0, helper_1.camelToKebab)(itemRelation);
            if (relationships[itemRelation]) {
                continue;
            }
            relationships[itemRelation] = {
                links: {
                    self: this.getLink(urlTable, data[idNameField], 'relationships', field),
                    related: this.getLink(urlTable, data[idNameField], field),
                },
            };
        }
        return {
            id: data[idNameField].toString(),
            type: urlTable,
            attributes,
            relationships,
            links: {
                self: this.getLink(urlTable, data[idNameField]),
            },
        };
    }
    transformInclude(data) {
        const result = {};
        for (const itemRow of data) {
            const needField = Object.keys(itemRow).filter((i) => this.relationColumns.has(i));
            for (const field of needField) {
                if (!itemRow[field] ||
                    (Array.isArray(itemRow[field]) && itemRow[field].lenght > 0)) {
                    continue;
                }
                result[field] = result[field] || {};
                const includeArray = Array.isArray(itemRow[field])
                    ? itemRow[field]
                    : [itemRow[field]];
                for (const includeItem of includeArray) {
                    const idName = this.relationPrimaryField.get(field);
                    const id = includeItem[idName];
                    if (result[field][id]) {
                        continue;
                    }
                    result[field][id] = includeItem;
                }
            }
        }
        return Object.keys(result).reduce((acum, item) => {
            const items = Object.values(result[item]);
            acum.push(...items.map((i) => {
                const dataForInclude = this.transformData(i, [], item);
                dataForInclude.type = (0, helper_1.camelToKebab)(i['constructor']['name']);
                return dataForInclude;
            }));
            return acum;
        }, []);
    }
    getPrefixArray() {
        const urlPath = [''];
        const prefix = this.applicationConfig.getGlobalPrefix();
        const version = this.applicationConfig.getVersioning();
        this.routePathFactory = new route_path_factory_1.RoutePathFactory(this.applicationConfig);
        if (prefix) {
            urlPath.push(this.applicationConfig.getGlobalPrefix());
        }
        if (version && version.type === common_1.VersioningType.URI) {
            const firstVersion = Array.isArray(version.defaultVersion)
                ? version.defaultVersion[0]
                : version.defaultVersion;
            if (firstVersion) {
                urlPath.push(`${this.routePathFactory.getVersionPrefix(version)}${firstVersion.toString()}`);
            }
        }
        return urlPath;
    }
    getLink(...partOfUrl) {
        const urlPath = this.getPrefixArray();
        urlPath.push(...partOfUrl);
        return urlPath.join('/');
    }
}
exports.TransformMixinService = TransformMixinService;
//# sourceMappingURL=transform.mixin.js.map